# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

XTRACT_STATS_METADATA = Metadata(
    id="e725669f09830b40ddd91ee1f55f1928db5f4c9e.boutiques",
    name="xtract_stats",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


XtractStatsParameters = typing.TypedDict('XtractStatsParameters', {
    "__STYX_TYPE__": typing.Literal["xtract_stats"],
    "folder_basename": str,
    "XTRACT_dir": str,
    "xtract2diff": str,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "xtract_stats": xtract_stats_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "xtract_stats": xtract_stats_outputs,
    }.get(t)


class XtractStatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `xtract_stats(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    csv_output: OutputPathType
    """CSV file containing the statistics from XTRACT analysis."""


def xtract_stats_params(
    folder_basename: str,
    xtract_dir: str,
    xtract2diff: str,
) -> XtractStatsParameters:
    """
    Build parameters.
    
    Args:
        folder_basename: Path to microstructure folder and basename of data\
            (e.g. /home/DTI/dti_).
        xtract_dir: Path to XTRACT output folder.
        xtract2diff: EITHER XTRACT results to diffusion space transform OR\
            'native' if tracts are already in diffusion space.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "xtract_stats",
        "folder_basename": folder_basename,
        "XTRACT_dir": xtract_dir,
        "xtract2diff": xtract2diff,
    }
    return params


def xtract_stats_cargs(
    params: XtractStatsParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("xtract_stats")
    cargs.extend([
        "-d",
        params.get("folder_basename")
    ])
    cargs.extend([
        "-xtract",
        params.get("XTRACT_dir")
    ])
    cargs.extend([
        "-w",
        params.get("xtract2diff")
    ])
    cargs.append("[reference]")
    cargs.append("[output_file]")
    cargs.append("[structures_file]")
    cargs.append("[threshold]")
    cargs.append("[measurements]")
    cargs.append("[keep_temp_files]")
    return cargs


def xtract_stats_outputs(
    params: XtractStatsParameters,
    execution: Execution,
) -> XtractStatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = XtractStatsOutputs(
        root=execution.output_file("."),
        csv_output=execution.output_file("[OUTPUT_FILE]"),
    )
    return ret


def xtract_stats_execute(
    params: XtractStatsParameters,
    execution: Execution,
) -> XtractStatsOutputs:
    """
    Quantitative evaluation tool of XTRACT results in neuroimaging.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `XtractStatsOutputs`).
    """
    params = execution.params(params)
    cargs = xtract_stats_cargs(params, execution)
    ret = xtract_stats_outputs(params, execution)
    execution.run(cargs)
    return ret


def xtract_stats(
    folder_basename: str,
    xtract_dir: str,
    xtract2diff: str,
    runner: Runner | None = None,
) -> XtractStatsOutputs:
    """
    Quantitative evaluation tool of XTRACT results in neuroimaging.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        folder_basename: Path to microstructure folder and basename of data\
            (e.g. /home/DTI/dti_).
        xtract_dir: Path to XTRACT output folder.
        xtract2diff: EITHER XTRACT results to diffusion space transform OR\
            'native' if tracts are already in diffusion space.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `XtractStatsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(XTRACT_STATS_METADATA)
    params = xtract_stats_params(
        folder_basename=folder_basename,
        xtract_dir=xtract_dir,
        xtract2diff=xtract2diff,
    )
    return xtract_stats_execute(params, execution)


__all__ = [
    "XTRACT_STATS_METADATA",
    "XtractStatsOutputs",
    "XtractStatsParameters",
    "xtract_stats",
    "xtract_stats_params",
]
